#include "fosfft.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define N (1<<16)
#define M (1<<8)

void test_equal_and_positive(
  int *retval, double beta1, double x1, double beta2, double x2
) {
  double y1 = fosfft_kaiser(beta1, x1);
  double y2 = fosfft_kaiser(beta2, x2);
  printf(
    "Check fosfft_kaiser(%.02f, %.02f) == fosfft_kaiser(%.02f, %.02f) > 0 ",
    beta1, x1, beta2, x2
  );
  if (!(isfinite(y1) && y1 > 0 && isfinite(y2) && y2 > 0) || y1 != y2) {
    printf("FAIL\n");
    *retval = 1;
  } else printf("OK\n");
}

void test_minimum(
  int *retval,
  fosfft_t *forward,
  complex double *src,
  complex double *dst,
  double blur
) {
  const double beta = M_PI * sqrt(blur*blur - 1);
  const int j = round(blur*M);
  double minimum;
  int i;
  for (i=0; i<M; i++) src[i] = fosfft_kaiser(beta, (2*i+1.0)/M - 1.0);
  fosfft(forward, src, dst, N);
  minimum = cabs(dst[j]);
  printf(
    "Kaiser window with main lobe width = 2 x %.02f (beta = %f) ",
    blur, beta
  );
  for (i=0; i<N && i<=j+1; i++) {
    if (i != j && cabs(dst[i]) <= minimum) {
      printf("FAIL\n");
      *retval = 1;
      return;
    }
  }
  printf("OK\n");
}

int main(int argc, char **argv) {
  int retval = 0;
  complex double *src, *dst;
  fosfft_t *forward;
  // test some corner cases and symmetry:
  test_equal_and_positive(&retval, 2.5, -1.0, 2.5, 1.0);
  test_equal_and_positive(&retval, 2.5, -0.8, 2.5, 0.8);
  test_equal_and_positive(&retval, 0.0, -1.0, 0.0, 0.0);
  test_equal_and_positive(&retval, 0.0, -0.3, 0.0, 0.0);
  test_equal_and_positive(&retval, 0.0, 1.0, 0.0, 0.0);
  // use FFT to check main lobe width in the frequency domain:
  forward = fosfft_init_forward(N);
  src = calloc(N, sizeof(*src));
  dst = calloc(N, sizeof(*src));
  test_minimum(&retval, forward, src, dst, 1.0);
  test_minimum(&retval, forward, src, dst, 2.0);
  test_minimum(&retval, forward, src, dst, 2.8);
  // return result:
  return retval;
}
